'use strict';

var test = require('tape');
var av = require('../index');

var schema = {
  type: 'array',
  items: {
    properties: {
      firstName: {
        properties: {
          value: {
            type: 'string',
            minLength: 2,
          }
        }

      },
      emailAddress: {
        properties: {
          value: {
            type: 'string',
            pattern: '^\\S+@\\S+$',
          },
        }
      },
      age: {
        properties: {
          value: {
            type: 'integer'
          }
        }
      }
    }
  }
}

var validObjects = [{
  firstName: {
    value: 'Oscar',
    placeholder: '',
    error: 'Plase enter a longer name',
    valid: true
  },
  emailAddress: {
    value: 'test@email.com',
    placeholder: '',
    error: 'Invalid email address',
    valid: true
  },
  age: {
    value: 5,
    placeholder: '',
    error: 'Age must be an integer',
    valid: true
  }
}];

var invalidObjects = [{
  firstName: {
    value: 'Oscar',
    placeholder: '',
    error: 'Plase enter a longer name',
    valid: true
  },
  emailAddress: {
    value: 'notanemailaddress',
    placeholder: '',
    error: 'Invalid email address',
    valid: true
  },
  age: {
    value: 5,
    placeholder: '',
    error: 'Age must be an integer',
    valid: true
  }
}];

test('not providing an array throws', t => {
  t.throws(() => {
    var result = av.validate(null, schema);
  });
  t.end();
})

test('false flatten parameter returns Boolean', t => {
  var result = av.validate(validObjects, schema, false);
  t.equal(typeof result === 'boolean', true);
  t.end();
});

test('no flatten parameter returns flattened objects', t => {
  var result = av.validate(validObjects, schema)[0];

  t.equal(result.firstName, 'Oscar');
  t.equal(result.emailAddress, 'test@email.com');
  t.equal(result.age, 5);
  t.end();
});

test('invalid data with flatten returns empty array', t => {
  var result = av.validate(invalidObjects, schema);

  t.deepEqual(result, []);
  t.end();
});

test('invalid data without flatten returns false', t => {
  var result = av.validate(invalidObjects, schema, false);

  t.equal(result, false);
  t.end();
});

test('setting pristine works as intended', t => {
  var result = av.validate(invalidObjects, schema, false);

  av.setPristine(invalidObjects);

  t.equal(invalidObjects[0].emailAddress.valid, true);
  t.end();
});

test('toValidationModel correctly transforms data', t => {
  var data = [{
    name: 'Test',
    age: 21
  }];

  var result = av.toValidationModel(data);

  var expected = [
    {
      name: {
        value: 'Test',
        placeholder: '',
        error: '',
        valid: true
      },
      age: {
        value: 21,
        placeholder: '',
        error: '',
        valid: true
      }
    }
  ];

  t.deepEqual(result, expected);
  t.end();
});

test('setPristine works on validationmodel', t => {
  var transformed = [
    {
      name: {
        value: 'Test',
        placeholder: '',
        error: '',
        valid: false
      },
      age: {
        value: 21,
        placeholder: '',
        error: '',
        valid: false
      }
    }
  ];

  av.setPristine(transformed);

  var expected = [
    {
      name: {
        value: 'Test',
        placeholder: '',
        error: '',
        valid: true
      },
      age: {
        value: 21,
        placeholder: '',
        error: '',
        valid: true
      }
    }
  ];

  t.deepEqual(transformed, expected);
  t.end();
});

test('setPristine doesnt apply to non-object properties', t => {
  var transformed = [
    {
      name: {
        value: 'Test',
        placeholder: '',
        error: '',
        valid: false
      },
      age: {
        value: 21,
        placeholder: '',
        error: '',
        valid: false
      },
      valid: true
    }
  ];

  av.setPristine(transformed);

  t.equal(transformed[0].valid, true);
  t.end();
});

test('flattenValues flattens array of objects', t => {
  var array = [
    {
      name: {
        value: 'Test',
        placeholder: '',
        error: '',
        valid: false
      },
      age: {
        value: 21,
        placeholder: '',
        error: '',
        valid: false
      }
    },
    {
      name: {
        value: 'Data',
        placeholder: '',
        error: '',
        valid: false
      },
      age: {
        value: 25,
        placeholder: '',
        error: '',
        valid: false
      }
    }
  ];

  var result = av.flattenValues(array);

  var expected = [
    {
      name: 'Test',
      age: 21
    }, 
    {
      name: 'Data',
      age: 25
    }
  ]

  t.deepEqual(result, expected);
  t.end();
})

test('flattenValue flattens single object', t => {
  var singleObject = {
    name: {
      value: 'Test',
      placeholder: '',
      error: '',
      valid: false
    },
    age: {
      value: 21,
      placeholder: '',
      error: '',
      valid: false
    }
  }

  var result = av.flattenValue(singleObject);

  var expected = {
    name: 'Test',
    age: 21
  };

  t.deepEqual(result, expected);
  t.end();
})

